///<reference path="../../node_modules/jquery/dist/jquery.min.js"/>
/**
* JQuery plugin for creating tooltips
* Copyright (c) 2018 Callisto (https://github.com/callisto2410/jq-tooltip)
*/
(function($) {
  'use strict';

  // Default Settings
  $.tooltipDefaults = {
    // Template
    template: '<div class="jq-tooltip">{content}</div>',

    // The content can be specified in an element attribute or passed as a string
    content: undefined,

    // Positioning
    // (top, top-left, top-right, right, right-top, right-bottom, bottom,
    // bottom-left, bottom-right, left, left-top, left-bottom)
    position: 'top',

    // Theme
    // (strawberry, orange, banana, lime, blueberry, cocoa, silver, slate)
    theme: 'slate',

    // The width, if set to 0, will be calculated automatically
    width: 0,

    // Get parameters from the attribute
    attr: true,

    // Attribute Name (data-tooltip)
    attrID: 'tooltip',

    // Gap
    gap: 8,

    // Name of the custom class
    class: undefined,

    // ID for stored data (tooltip.data)
    dataID: 'tooltip.data'
  };

  var BODY = $('body');
  var TARGET = {};
  var TOOLTIP = {};

  var fn = {
    target: {
      // Initializing the target element
      init: function () {
        TARGET.width = TARGET.pointer.outerWidth();
        TARGET.height = TARGET.pointer.outerHeight();
        TARGET.top = TARGET.pointer.offset().top;
        TARGET.left = TARGET.pointer.offset().left;

        return this;
      }
    },

    tooltip: {
      // Creating a tooltip
      create: function () {
        TOOLTIP.pointer = $(TOOLTIP.options.template);
        fn.tooltip.setContent().setData();
        BODY.append(TOOLTIP.pointer);

        return this;
      },

      // Inserting content
      setContent: function () {
        TOOLTIP.pointer.html(TOOLTIP.options.content);

        return this;
      },

      // Setting Data
      setData: function () {
        TOOLTIP.pointer.attr('data-theme', TOOLTIP.options.theme);
        TOOLTIP.pointer.attr('data-position', TOOLTIP.options.position);

        return this;
      },

      // Initializing the tooltip
      init: function () {
        TOOLTIP.width = (TOOLTIP.options.width <= 0) ? TOOLTIP.pointer.outerWidth() : TOOLTIP.options.width;
        TOOLTIP.height = TOOLTIP.pointer.outerHeight();

        switch (TOOLTIP.options.position) {
          case 'top':
          TOOLTIP.top = TARGET.top - TOOLTIP.height - TOOLTIP.options.gap;
          TOOLTIP.left = TARGET.left - ((TOOLTIP.width - TARGET.width) / 2);
          break;

          case 'top-left':
          TOOLTIP.top = TARGET.top - TOOLTIP.height - TOOLTIP.options.gap;
          TOOLTIP.left = TARGET.left;
          break;

          case 'top-right':
          TOOLTIP.top = TARGET.top - TOOLTIP.height - TOOLTIP.options.gap;
          TOOLTIP.left = TARGET.left - (TOOLTIP.width - TARGET.width);
          break;


          case 'right':
          TOOLTIP.top = TARGET.top - ((TOOLTIP.height - TARGET.height) / 2);
          TOOLTIP.left = TARGET.left + TARGET.width + TOOLTIP.options.gap;
          break;

          case 'right-top':
          TOOLTIP.top = TARGET.top;
          TOOLTIP.left = TARGET.left + TARGET.width + TOOLTIP.options.gap;
          break;

          case 'right-bottom':
          TOOLTIP.top = TARGET.top + (TARGET.height - TOOLTIP.height);
          TOOLTIP.left = TARGET.left + TARGET.width + TOOLTIP.options.gap;
          break;


          case 'bottom':
          TOOLTIP.top = TARGET.top + TARGET.height + TOOLTIP.options.gap;
          TOOLTIP.left = TARGET.left - ((TOOLTIP.width - TARGET.width) / 2);
          break;

          case 'bottom-left':
          TOOLTIP.top = TARGET.top + TARGET.height + TOOLTIP.options.gap;
          TOOLTIP.left = TARGET.left;
          break;

          case 'bottom-right':
          TOOLTIP.top = TARGET.top + TARGET.height + TOOLTIP.options.gap;
          TOOLTIP.left = TARGET.left - (TOOLTIP.width - TARGET.width);
          break;


          case 'left':
          TOOLTIP.top = TARGET.top - ((TOOLTIP.height - TARGET.height) / 2);
          TOOLTIP.left = TARGET.left - TOOLTIP.width - TOOLTIP.options.gap;
          break;

          case 'left-top':
          TOOLTIP.top = TARGET.top;
          TOOLTIP.left = TARGET.left - TOOLTIP.width - TOOLTIP.options.gap;
          break;

          case 'left-bottom':
          TOOLTIP.top = TARGET.top + (TARGET.height - TOOLTIP.height);
          TOOLTIP.left = TARGET.left - TOOLTIP.width - TOOLTIP.options.gap;
          break;


          default:
          TOOLTIP.top = TARGET.top - TOOLTIP.height - TOOLTIP.options.gap;
          TOOLTIP.left = TARGET.left - ((TOOLTIP.width - TARGET.width) / 2);
          break;
        }

        TOOLTIP.pointer.css({
          top: TOOLTIP.top,
          left: TOOLTIP.left,
          width: TOOLTIP.width
        });

        TOOLTIP.pointer.data(TOOLTIP.options.dataID, TOOLTIP.options);

        return this;
      },

      // Display
      show: function () {
        TOOLTIP.pointer.addClass('tooltip-active');

        if ( TOOLTIP.options.class !== undefined ) {
          TOOLTIP.pointer.addClass(TOOLTIP.options.class);
        }

        var data = TARGET.pointer.data(TOOLTIP.options.dataID);

        if ( $.type(data) === 'array' ) {
          data.push(TOOLTIP.pointer);
        } else {
          data = [TOOLTIP.pointer];
        }

        TARGET.pointer.data(TOOLTIP.options.dataID, data);

        return this;
      },

      // Update settings
      update: function ( elem, opt ) {
        elem.each(function () {
          if ( TOOLTIP.options === undefined ) return;

          var target = $(this);
          var pointer = target.data(TOOLTIP.options.dataID);

          if ( pointer === undefined ) return;

          fn.target.init();

          $.each(pointer, function ( i ) {
            var tp = pointer[i];
            var data = tp.data(TOOLTIP.options.dataID);
            TOOLTIP.options = $.extend(data, opt || {});

            tp.data(TOOLTIP.options.dataID, TOOLTIP.options);
            fn.tooltip.setContent().setData().init();
          });
        });

        return this;
      },

      // Close the tooltip
      close: function ( elem ) {
        elem.each(function () {
          if ( TOOLTIP.options === undefined ) return;

          var target = $(this);
          var pointer = target.data(TOOLTIP.options.dataID);

          if ( pointer === undefined ) return;

          $.each(pointer, function ( i ) {
            var tp = pointer[i];
            var delay = parseFloat(tp.css('transition-duration')) * 1000;

            tp.removeClass('tooltip-active');
            setTimeout(function () {
              tp.remove();
            }, delay);
          });

          target.removeData(TOOLTIP.options.dataID);
        });

        return this;
      }
    }
  };


  // $.fn.tooltip implementation
  $.fn.tooltip = function( opt, obj ) {
    var t = $(this);
    // If parameters are passed as a string
    if ( $.type(opt) === "string" ) {
      switch ( opt ) {
        // Update settings
        case 'update':
        $.tooltip[opt](t, obj);
        break;

        // Close the tooltip
        case 'close':
        $.tooltip[opt](t);
        break;

        // Inserting content, undo reading parameters from an attribute
        default:
        $.tooltip(t, {
          content: opt,
          attr: false,
        });
        break;
      }
    } else {
      $.tooltip(t, opt);
    }
    return t;
  };


  // $.tooltip implementation
  $.tooltip = function ( elem, opt ) {
    elem.each(function () {
      TARGET.pointer = $(this);
      TOOLTIP.options = $.extend({}, $.tooltipDefaults, opt || {});

      fn.target.init();

      // If the parameters are obtained from the attribute
      if ( TOOLTIP.options.attr ) {
        var data = TARGET.pointer.data(TOOLTIP.options.attrID);

        switch ( $.type(data) ) {
          // If an array of parameters is specified,
          // then you need to create several tooltips
          case 'array':
          $.each(data, function ( i ) {
            data[i].attr = false;
            TARGET.pointer.tooltip(data[i]);
          });
          break;

          // If an object is specified,
          // update the default settings for the tooltip
          case 'object':
          TOOLTIP.options = $.extend(TOOLTIP.options, data);
          fn.tooltip.create().init().show();
          break;

          // By default, the string is specified
          default:
          TOOLTIP.options.content = TARGET.pointer.data(TOOLTIP.options.attrID);
          fn.tooltip.create().init().show();
          break;
        }
      } else { // Then just run the tooltip
        fn.tooltip.create().init().show();
      }
    });
  };


  // $.tooltip.update implementation
  $.tooltip.update = function ( elem, opt ) {
    fn.tooltip.update(elem, opt);
  };


  // $.tooltip.close implementation
  $.tooltip.close = function ( elem ) {
    fn.tooltip.close(elem);
  };

})(jQuery);